/*
 * Pure JavaScript Browser Environment
 *   By John Resig <http://ejohn.org/>
 * Copyright 2008 John Resig, under the MIT License
 */

// The window Object
var window = this;

(function(){

  // Browser Navigator

  window.navigator = {
    get userAgent(){
      return "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.3) Gecko/20070309 Firefox/2.0.0.3";
    }
  };

  var curLocation = (new java.io.File("./")).toURL();

  window.__defineSetter__("location", function(url){
    var xhr = new XMLHttpRequest();
    xhr.open("GET", url);
    xhr.onreadystatechange = function(){
      curLocation = new java.net.URL( curLocation, url );
      window.document = xhr.responseXML;

      var event = document.createEvent();
      event.initEvent("load");
      window.dispatchEvent( event );
    };
    xhr.send();
  });

  window.__defineGetter__("location", function(url){
    return {
      get protocol(){
        return curLocation.getProtocol() + ":";
      },
      get href(){
        return curLocation.toString();
      },
      toString: function(){
        return this.href;
      }
    };
  });

  // Timers

  var timers = [];

  window.setTimeout = function(fn, time){
    debug('window : setTimeout');
    var num;
    return num = setInterval(function(){
      fn();
      clearInterval(num);
    }, time);
  };

  window.setInterval = function(fn, time){
    debug('window : setInterval');
    var num = timers.length;

    timers[num] = new java.lang.Thread(new java.lang.Runnable({
      run: function(){
        while (true){
          java.lang.Thread.currentThread().sleep(time);
          fn();
        }
      }
    }));

    timers[num].start();

    return num;
  };

  window.clearInterval = function(num){
    debug('window : clearInterval');
    if ( timers[num] ) {
      timers[num].stop();
      delete timers[num];
    }
  };

  // Window Events

  var events = [{}];

  window.addEventListener = function(type, fn){
    debug('window : addEventListener');
    if ( !this.uuid || this == window ) {
      this.uuid = events.length;
      events[this.uuid] = {};
    }

    if ( !events[this.uuid][type] )
      events[this.uuid][type] = [];

    if ( events[this.uuid][type].indexOf( fn ) < 0 )
      events[this.uuid][type].push( fn );
  };

  window.removeEventListener = function(type, fn){
    debug('window : removeEventListener');
     if ( !this.uuid || this == window ) {
         this.uuid = events.length;
         events[this.uuid] = {};
     }

     if ( !events[this.uuid][type] )
      events[this.uuid][type] = [];

    events[this.uuid][type] =
      events[this.uuid][type].filter(function(f){
        return f != fn;
      });
  };

  window.dispatchEvent = function(event){
    debug('window : dispatchEvent');
    if ( event.type ) {
      if ( this.uuid && events[this.uuid][event.type] ) {
        var self = this;

        events[this.uuid][event.type].forEach(function(fn){
          fn.call( self, event );
        });
      }

      if ( this["on" + event.type] )
        this["on" + event.type].call( self, event );
    }
  };

  function Range(node) {
    this._dom = node;
  };

  Range.prototype = {
    setStart : function(el, offset) {
        debug('Range : setStart');
        this.startContainer = el;
    },
    setEnd : function(el, offset) {
        debug('Range : setEnd');
        this.endContainer = el;
    },
    setStartBefore : function(el) {
        debug('Range : setStartBefore');
        this.startContainer = el;
    },
    setStartAfter : function(el) {
        debug('Range : setStartAfter');
      this.startOffset = el;
    },
    setEndBefore : function(el) {
        debug('Range : setEndBefore');
      this.endContainer = el;
    },
    setEndAfter : function(el) {
        debug('Range : setEndAfter');
      this.endOffset = el;
    },
    selectNode : function() {
      print('selectNode function unimplemented');
    },
    selectNodeContents : function() {
      print('selectNodeContents function unimplemented');
    },
    collapse : function() {
        debug('Range : collapse');
        this.collapsed = true;
    },
    cloneContents : function() {
      print('cloneContents function unimplemented');
    },
    deleteContents : function() {
      print('deleteContents function unimplemented');
    },
    extractContents : function() {
      print('extractContents function unimplemented');
    },
    insertNode : function() {
      print('insertNode function unimplemented');
    },
    surroundContents : function() {
      print('surroundContents function unimplemented');
    },
    compareBoundaryPoints : function() {
      print('compareBoundaryPoints function unimplemented');
    },
    cloneRange : function() {
      print('cloneRange function unimplemented');
    },
    detach : function() {
      print('detach function unimplemented');
    },
    toString : function() {
      print('toString function unimplemented');
    },
    comparePoint : function() {
      print('comparePoint function unimplemented');
    },
    createContextualFragment : function(html) {
        debug('Range : createContextualFragment : html : ' + html);
        var stringFrag = new java.lang.String(html);
        var bytes = new java.io.ByteArrayInputStream(stringFrag.getBytes());
        var doc = Packages.javax.xml.parsers.
            DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(bytes);
        var newDoc = doc.getDocumentElement();
        var myNode = {
            _dom : newDoc
        };
        return this._dom.importNode(myNode, true);
    },
    isPointInRange : function() {}
  };

  // DOM Document

  window.DOMDocument = function(file){
    this._file = file;
    this._dom = Packages.javax.xml.parsers.
      DocumentBuilderFactory.newInstance()
        .newDocumentBuilder().parse(file);

    if ( !obj_nodes.containsKey( this._dom ) )
      obj_nodes.put( this._dom, this );
  };

  DOMDocument.prototype = {
    get nodeType(){
        debug('DOMDocument : nodeType');
      return 9;
    },
    createRange : function() {
        debug('DOMDocument : createRange');
        return new Range(this);
    },
    createTextNode: function(text){
        debug('DOMDocument : createTextNode : text : ' + text);
      return makeNode( this._dom.createTextNode(
        text.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;")) );
    },
    createElement: function(name){
        debug('DOMDocument : createElement(name) : ' + name);
      return makeNode( this._dom.createElement(name.toLowerCase()) );
    },
    getElementsByTagName: function(name){
        debug('DOMDocument : getElementsByTagName(name) : name : ' + name);
      return new DOMNodeList( this._dom.getElementsByTagName(
        name.toLowerCase()) );
    },
    getElementsByName: function(name){
        debug('DOMDocument : getElementsByName(name) : name : ' + name);
      var elems = this._dom.getElementsByTagName("*"), ret = [];
      ret.item = function(i){ return this[i]; };
      ret.getLength = function(){ return this.length; };

      for ( var i = 0; i < elems.length; i++ ) {
        var elem = elems.item(i);
        if ( elem.getAttribute("name") == name )
          ret.push( elem );
      }

      return new DOMNodeList( ret );
    },
    getElementById: function(id){
        debug('DOMDocument : getElementById : id : ' + id);
      var elems = this._dom.getElementsByTagName("*");

      for ( var i = 0; i < elems.length; i++ ) {
        var elem = elems.item(i);
        var elemId = elem.getAttribute("id");
        debug("elements include : " + elemId);
        if ( elemId == id ) {
          return makeNode(elem);
        }
      }

      return null;
    },
    get body(){
        debug('DOMDocument : body');
      return this.getElementsByTagName("body")[0];
    },
    get documentElement(){
        debug('DOMDocument : documentElement');
      return makeNode( this._dom.getDocumentElement() );
    },
    get ownerDocument(){
      print('calling unimplemented function ownerDocument');
      return null;
    },
    addEventListener: window.addEventListener,
    removeEventListener: window.removeEventListener,
    dispatchEvent: window.dispatchEvent,
    get nodeName() {
      return "#document";
    },
    importNode: function(node, deep){
        debug('DOMDocument : importNode : node : ' + node + ' : deep : ' + deep);
      return makeNode( this._dom.importNode(node._dom, deep) );
    },
    toString: function(){
      return "Document" + (typeof this._file == "string" ?
        ": " + this._file : "");
    },
    get innerHTML(){
        debug('DOMDocument : innerHTML');
      return this.documentElement.outerHTML;
    },

    get defaultView(){
        debug('DOMDocument : defaultView');
      return {
        getComputedStyle: function(elem){
          return {
            getPropertyValue: function(prop){
              prop = prop.replace(/\-(\w)/g,function(m,c){
                return c.toUpperCase();
              });
              var val = elem.style[prop];

              if ( prop == "opacity" && val == "" )
                val = "1";

              return val;
            }
          };
        }
      };
    },

    createEvent: function(){
        debug('DOMDocument : createEvent');
      return {
        type: "",
        initEvent: function(type){
          this.type = type;
        }
      };
    }
  };

  function getDocument(node){
    debug('DOMDocument : getDocument');
    return obj_nodes.get(node);
  }

  // DOM NodeList

  window.DOMNodeList = function(list){
    this._dom = list;
    this.length = list.getLength();

    for ( var i = 0; i < this.length; i++ ) {
      var node = list.item(i);
      this[i] = makeNode( node );
    }
  };

  DOMNodeList.prototype = {
    toString: function(){
      return "[ " +
        Array.prototype.join.call( this, ", " ) + " ]";
    },
    get outerHTML(){
        debug('DOMNodeList : outerHTML');
      return Array.prototype.map.call(
        this, function(node){return node.outerHTML;}).join('');
    }
  };

  // DOM Node

  window.DOMNode = function(node){
    this._dom = node;
  };

  DOMNode.prototype = {
    get nodeType(){
        debug('DOMNode : nodeType');
      return this._dom.getNodeType();
    },
    get nodeValue(){
        debug('DOMNode : nodeValue');
      return this._dom.getNodeValue();
    },
    get nodeName() {
        debug('DOMNode : nodeName');
      return this._dom.getNodeName();
    },
    get childNodes(){
        debug('DOMNode : childNodes');
      return new DOMNodeList( this._dom.getChildNodes() );
    },
    cloneNode: function(deep){
        debug('DOMNode : cloneNode : deep : ' + deep);
      return makeNode( this._dom.cloneNode(deep) );
    },
    get ownerDocument(){
        debug('DOMNode : ownerDocument');
      return getDocument( this._dom.ownerDocument );
    },
    get documentElement(){
        debug('DOMNode : documentElement');
      return makeNode( this._dom.documentElement );
    },
    get parentNode() {
        debug('DOMNode : parentNode');
      return makeNode( this._dom.getParentNode() );
    },
    get nextSibling() {
        debug('DOMNode : nextSibling');
      return makeNode( this._dom.getNextSibling() );
    },
    get previousSibling() {
        debug('DOMNode : previousSibling');
      return makeNode( this._dom.getPreviousSibling() );
    },
    toString: function(){
      return '"' + this.nodeValue + '"';
    },
    get outerHTML(){
        debug('DOMNode : outerHTML');
      return this.nodeValue;
    }
  };

  window.DOMComment = function(node){
    this._dom = node;
  };

  DOMComment.prototype = extend(new DOMNode(), {
    get nodeType(){
        debug('DOMComment : nodeType');
      return 8;
    },
    get outerHTML(){
        debug('DOMComment : outerHTML');
      return "<!--" + this.nodeValue + "-->";
    }
  });

  // DOM Element

  window.DOMElement = function(elem){
    this._dom = elem;
    this.style = {
      get opacity(){ return this._opacity; },
      set opacity(val){ this._opacity = val + ""; }
    };

    // Load CSS info
    var styles = (this.getAttribute("style") || "").split(/\s*;\s*/);

    for ( var i = 0; i < styles.length; i++ ) {
      var style = styles[i].split(/\s*:\s*/);
      if ( style.length == 2 )
        this.style[ style[0] ] = style[1];
    }

    if ( this.nodeName == "FORM" ) {
      this.__defineGetter__("elements", function(){
        return this.getElementsByTagName("*");
      });

      this.__defineGetter__("length", function(){
        var elems = this.elements;
        for ( var i = 0; i < elems.length; i++ ) {
          this[i] = elems[i];
        }

        return elems.length;
      });
    }

    if ( this.nodeName == "SELECT" ) {
      this.__defineGetter__("options", function(){
        return this.getElementsByTagName("option");
      });
    }

    this.defaultValue = this.value;
  };

  DOMElement.prototype = extend( new DOMNode(), {
    get nodeName(){
        debug('DOMElement : nodeName');
      return this.tagName;
    },
    get tagName(){
        debug('DOMElement : tagName: ' + this._dom.getTagName().toUpperCase());
      return this._dom.getTagName().toUpperCase();
    },
    toString: function(){
      return "<" + this.tagName + (this.id ? "#" + this.id : "" ) + ">";
    },
    get outerHTML(){
        debug('DOMElement : outerHTML');
      var ret = "<" + this.tagName, attr = this.attributes;

      for ( var i in attr )
        ret += " " + i + "='" + attr[i] + "'";

      if ( this.childNodes.length || this.nodeName == "SCRIPT" )
        ret += ">" + this.childNodes.outerHTML +
          "</" + this.tagName + ">";
      else
        ret += "/>";

      return ret;
    },

    get attributes(){
        debug('DOMElement : attributes');
      var attr = {}, attrs = this._dom.getAttributes();

      for ( var i = 0; i < attrs.getLength(); i++ )
        attr[ attrs.item(i).nodeName ] = attrs.item(i).nodeValue;

      return attr;
    },

    get innerHTML(){
        debug('DOMElement : innerHTML');
      return this.childNodes.outerHTML;
    },
    set innerHTML(html){
        debug('DOMElement : innerHTML(html) : html : ' + html);
      html = html.replace(/<\/?([A-Z]+)/g, function(m){
        return m.toLowerCase();
      }).replace(/&nbsp;/g, " ");

      var nodes = this.ownerDocument.importNode(
        new DOMDocument( new java.io.ByteArrayInputStream(
          (new java.lang.String("<wrap>" + html + "</wrap>"))
            .getBytes("UTF8"))).documentElement, true).childNodes;

      while (this.firstChild) {
        debug("removing child: " + this.firstChild);
        this.removeChild( this.firstChild );
      }

      for ( var i = 0; i < nodes.length; i++ )
        this.appendChild( nodes[i] );
    },

    get textContent(){
        debug('DOMElement : textContent');
      return nav(this.childNodes);

      function nav(nodes){
        var str = "";
        for ( var i = 0; i < nodes.length; i++ )
          if ( nodes[i].nodeType == 3 )
            str += nodes[i].nodeValue;
          else if ( nodes[i].nodeType == 1 )
            str += nav(nodes[i].childNodes);
        return str;
      }
    },
    set textContent(text){
        debug('DOMElement : textContent(text) : text : ' + text);
      while (this.firstChild) {
        debug("removing child : " + this.firstChild);
        this.removeChild( this.firstChild );
      }
      this.appendChild( this.ownerDocument.createTextNode(text));
    },

    style: {},
    clientHeight: 0,
    clientWidth: 0,
    offsetHeight: 0,
    offsetWidth: 0,

    get disabled() {
        debug('DOMElement : disabled');
      var val = this.getAttribute("disabled");
      return val != "false" && !!val;
    },
    set disabled(val) {
      debug('DOMElement : disabled(val) : val : ' + val);
      return this.setAttribute("disabled",val);
    },

    get checked() {
      debug('DOMElement : checked');
      var val = this.getAttribute("checked");
      return val != "false" && !!val;
    },
    set checked(val) {
      debug('DOMElement : checked(val) : val : ' + val);
      return this.setAttribute("checked",val);
    },

    get selected() {
        debug('DOMElement : selected');
      if ( !this._selectDone ) {
        this._selectDone = true;

        if ( this.nodeName == "OPTION" && !this.parentNode.getAttribute("multiple") ) {
          var opt = this.parentNode.getElementsByTagName("option");

          if ( this == opt[0] ) {
            var select = true;

            for ( var i = 1; i < opt.length; i++ )
              if ( opt[i].selected ) {
                select = false;
                break;
              }

            if ( select )
              this.selected = true;
          }
        }
      }

      var val = this.getAttribute("selected");
      return val != "false" && !!val;
    },
    set selected(val) {
      debug('DOMElement : selected(val) : val : ' + val);
      return this.setAttribute("selected",val);
    },

    get className() {
      debug('DOMElement : className');
      return this.getAttribute("class") || "";
    },
    set className(val) {
      debug('DOMElement : className(val) : val : ' + val);
      return this.setAttribute("class",
        val.replace(/(^\s*|\s*$)/g,""));
    },

    get type() {
      debug('DOMElement : type');
      return this.getAttribute("type") || "";
    },
    set type(val) {
      debug('DOMElement : type(val) : val : ' + val);
      return this.setAttribute("type",val);
    },

    get defaultValue() {
      debug('DOMElement : defaultValue');
      return this.getAttribute("defaultValue") || "";
    },
    set defaultValue(val) {
      debug('DOMElement : defaultValue(val) : val : ' + val);
      return this.setAttribute("defaultValue",val);
    },
    get value() {
      debug('DOMElement : value');
      return this.getAttribute("value") || "";
    },
    set value(val) {
      debug('DOMElement : value(val) : val : ' + val);
      return this.setAttribute("value",val);
    },

    get src() {
      debug('DOMElement : src');
      return this.getAttribute("src") || "";
    },
    set src(val) {
      debug('DOMElement : src(val) : val : ' + val);
      return this.setAttribute("src",val);
    },

    get id() {
      debug('DOMElement : id');
      return this.getAttribute("id") || "";
    },
    set id(val) {
      debug('DOMElement : id(val) : val : ' + val);
      return this.setAttribute("id",val);
    },

    getAttribute: function(name){
        debug('DOMElement : getAttribute(name): ' + name);
      return this._dom.hasAttribute(name) ?
        new String( this._dom.getAttribute(name) ) :
        null;
    },
    setAttribute: function(name,value){
        debug('DOMElement : setAttribute(name, value): name : ' + name + ', value : ' + value);
      this._dom.setAttribute(name,value);
    },
    removeAttribute: function(name){
        debug('DOMElement : removeAttribute: name : ' + name);
      this._dom.removeAttribute(name);
    },

    get childNodes(){
        debug('DOMElement : childNodes');
      return new DOMNodeList( this._dom.getChildNodes() );
    },
    get firstChild(){
        debug('DOMElement : firstChild : id :' + this._dom.getAttribute("id") );
      return makeNode( this._dom.getFirstChild() );
    },
    get lastChild(){
        debug('DOMElement : lastChild : id : ' + this._dom.getAttribute("id") );
      return makeNode( this._dom.getLastChild() );
    },
    appendChild: function(node){
        debug('DOMElement : appendChild : ' + node);
      var appendedElement= this._dom.appendChild( node._dom );
      return makeNode(appendedElement);
    },
    insertBefore: function(node,before){
        debug('DOMElement : insertBefore');
      this._dom.insertBefore( node._dom, before ? before._dom : before );

      execScripts( node );

      function execScripts( node ) {
        if ( node.nodeName == "SCRIPT" ) {
          if ( !node.getAttribute("src") ) {
            eval.call( window, node.textContent );
          }
        } else {
          var scripts = node.getElementsByTagName("script");
          for ( var i = 0; i < scripts.length; i++ ) {
            execScripts( node );
          }
        }
      }
    },
    removeChild: function(node){
        debug('DOMElement : removeChild');
      this._dom.removeChild( node._dom );
    },

    getElementsByTagName: DOMDocument.prototype.getElementsByTagName,

    addEventListener: window.addEventListener,
    removeEventListener: window.removeEventListener,
    dispatchEvent: window.dispatchEvent,

    click: function(){
        debug('DOMElement : click');
      var event = document.createEvent();
      event.initEvent("click");
      this.dispatchEvent(event);
    },
    submit: function(){
        debug('DOMElement : submit');
      var event = document.createEvent();
      event.initEvent("submit");
      this.dispatchEvent(event);
    },
    focus: function(){
        debug('DOMElement : focus');
      var event = document.createEvent();
      event.initEvent("focus");
      this.dispatchEvent(event);
    },
    blur: function(){
        debug('DOMElement : blur');
      var event = document.createEvent();
      event.initEvent("blur");
      this.dispatchEvent(event);
    },
    get contentWindow(){
        debug('DOMElement : contentWindow');
      return this.nodeName == "IFRAME" ? {
        document: this.contentDocument
      } : null;
    },
    get contentDocument(){
        debug('DOMElement : contentDocument');
      if ( this.nodeName == "IFRAME" ) {
        if ( !this._doc )
          this._doc = new DOMDocument(
            new java.io.ByteArrayInputStream((new java.lang.String(
            "<html><head><title></title></head><body></body></html>"))
            .getBytes("UTF8")));
        return this._doc;
      } else
        return null;
    }
  });

  // Helper method for extending one object with another

  function extend(a,b) {
    for ( var i in b ) {
      var g = b.__lookupGetter__(i), s = b.__lookupSetter__(i);

      if ( g || s ) {
        if ( g )
          a.__defineGetter__(i, g);
        if ( s )
          a.__defineSetter__(i, s);
      } else
        a[i] = b[i];
    }
    return a;
  }

  // Helper method for generating the right
  // DOM objects based upon the type

  var obj_nodes = new java.util.HashMap();

  function makeNode(node){

    if ( node ) {
      if ( !obj_nodes.containsKey( node ) ) {
        if(node.getNodeType() == 1) {
          obj_nodes.put(node, new DOMElement(node));
        } else if (node.getNodeType() == 8 ) {
          obj_nodes.put(node, new DOMComment(node));
        } else {
          obj_nodes.put(node, new DOMNode(node));
        }
      }
      return obj_nodes.get(node);
    } else {
      return null;
    }
  }

  // XMLHttpRequest
  // Originally implemented by Yehuda Katz

  window.XMLHttpRequest = function(){
    this.headers = {};
    this.responseHeaders = {};
  };

  XMLHttpRequest.prototype = {
    open: function(method, url, async, user, password){
      this.readyState = 1;
      if (async)
        this.async = true;
      this.method = method || "GET";
      this.url = url;
      this.onreadystatechange();
    },
    setRequestHeader: function(header, value){
      this.headers[header] = value;
    },
    getResponseHeader: function(header){ },
    send: function(data){
      var self = this;

      function makeRequest(){
        var url = new java.net.URL(curLocation, self.url);

        if ( url.getProtocol() == "file" ) {
          if ( self.method == "PUT" ) {
            var out = new java.io.FileWriter(
                new java.io.File( new java.net.URI( url.toString() ) ) ),
              text = new java.lang.String( data || "" );

            out.write( text, 0, text.length() );
            out.flush();
            out.close();
          } else if ( self.method == "DELETE" ) {
            var file = new java.io.File( new java.net.URI( url.toString() ) );
            file["delete"]();
          } else {
            var connection = url.openConnection();
            connection.connect();
            handleResponse();
          }
        } else {
          var connection = url.openConnection();

          connection.setRequestMethod( self.method );

          // Add headers to Java connection
          for (var header in self.headers)
            connection.addRequestProperty(header, self.headers[header]);

          connection.connect();

          // Stick the response headers into responseHeaders
          for (var i = 0; ; i++) {
            var headerName = connection.getHeaderFieldKey(i);
            var headerValue = connection.getHeaderField(i);
            if (!headerName && !headerValue) break;
            if (headerName)
              self.responseHeaders[headerName] = headerValue;
          }

          handleResponse();
        }

        function handleResponse(){
          self.readyState = 4;
          self.status = parseInt(connection.responseCode) || undefined;
          self.statusText = connection.responseMessage || "";

          var contentEncoding = connection.getContentEncoding() || "utf-8",
            stream = (contentEncoding.equalsIgnoreCase("gzip") || contentEncoding.equalsIgnoreCase("decompress") )?
                     new java.util.zip.GZIPInputStream(connection.getInputStream()) :
                     connection.getInputStream(),
            baos = new java.io.ByteArrayOutputStream(),
                   buffer = java.lang.reflect.Array.newInstance(java.lang.Byte.TYPE, 1024),
            length,
            responseXML = null;

          while ((length = stream.read(buffer)) != -1) {
            baos.write(buffer, 0, length);
          }

          baos.close();
          stream.close();

          self.responseText = java.nio.charset.Charset.forName(contentEncoding)
            .decode(java.nio.ByteBuffer.wrap(baos.toByteArray())).toString();

          self.__defineGetter__("responseXML", function(){
            return responseXML;
          });

          if ( self.responseText.match(/^\s*</) ) {
            try {
              responseXML = new DOMDocument(
                new java.io.ByteArrayInputStream(
                  (new java.lang.String(
                    self.responseText)).getBytes("UTF8")));
            } catch(e) {}
          }
        }

        self.onreadystatechange();
      }

      if (this.async)
        (new java.lang.Thread(new java.lang.Runnable({
          run: makeRequest
        }))).start();
      else
        makeRequest();
    },
    abort: function(){},
    onreadystatechange: function(){},
    getResponseHeader: function(header){
      if (this.readyState < 3)
        throw new Error("INVALID_STATE_ERR");
      else {
        var returnedHeaders = [];
        for (var rHeader in this.responseHeaders) {
          if (rHeader.match(new RegExp(header, "i")))
            returnedHeaders.push(this.responseHeaders[rHeader]);
        }

        if (returnedHeaders.length)
          return returnedHeaders.join(", ");
      }

      return null;
    },
    getAllResponseHeaders: function(header){
      if (this.readyState < 3)
        throw new Error("INVALID_STATE_ERR");
      else {
        var returnedHeaders = [];

        for (var header in this.responseHeaders)
          returnedHeaders.push( header + ": " + this.responseHeaders[header] );

        return returnedHeaders.join("\r\n");
      }
    },
    async: true,
    readyState: 0,
    responseText: "",
    status: 0
  };
})();
