Ext.namespace("validators");

validators.field = function(){

    var parseDate = function(val){
        var altFormats = ['Y-m-d', 'm\/Y', 'm-Y'];
        var v = Date.parseDate(val, 'm/d/Y');
        
        if (!v) {
            for (var i = 0, len = altFormats.length; i < len && !v; i++) {
                v = Date.parseDate(val, altFormats[i]);
            }
        }
        
        return v;
    };
    
    /* Luhn algorithm number checker - (c) 2005-2008 shaman - www.planzero.org *
     * This code has been released into the public domain, however please      *
     * give credit to the original author where possible.                      */
    var luhnCheck = function luhn_check(number){
    
        // Set the string length and parity
        var number_length = number.length;
        var parity = number_length % 2;
        
        // Loop through each digit and do the maths
        var total = 0;
        for (i = 0; i < number_length; i++) {
            var digit = number.charAt(i);
            // Multiply alternate digits by two
            if (i % 2 == parity) {
                digit = digit * 2;
                // If the sum is two digits, add them together (in effect)
                if (digit > 9) {
                    digit = digit - 9;
                }
            }
            // Total up the digits
            total = total + parseInt(digit);
        }
        
        // If the total mod 10 equals 0, the number is valid
        if (total % 10 == 0) {
            return true;
        }
        else {
            return false;
        }
    };
    
    return {
    
        dateAfterValidator: function(val){
            var valid = false;
            
            if (val === '') {
                valid = true;
            }
            
            // Continue with validation
            var enteredDate = parseDate(val);
            
            if (!!enteredDate) {
                var todayDate = new Date();
                
                // If the date is not before or today's date, then there is an error (only validate valid dates)
                if (enteredDate > todayDate) {
                
                    valid = true;
                }
                else {
                    valid = false;
                }
            }
            
            return valid;
        },
        
        creditCardNumberValidator: function(val){
            var valid = false;
            
            if (val === '' || val.match(/^[0-9]+$/)) {
                // Check against the LUHN formula
                if (!luhnCheck(val)) {
                    valid = false;
                }
                else {
                    valid = true;
                }
            }
            else {
                valid = false;
            }
            
            return valid;
        }
        
    };
};

