/**
 * Superclass for executing junit tests in the jvm using Rhino
 */
package com.objectpartners.envjs;

import junit.framework.TestCase;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ScriptableObject;


/**
 * @author Jeff Hemminger
 *
 */
public abstract class AbstractRhinoTestCase extends TestCase {
    private Context context;
    private Shell shell;
    public static final String BASE_SCRIPT_DIR = "test/com/objectpartners/envjs/";
    
    /**
     * Configure context and shell in setup.
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        context = Context.enter();
        try {
            /**
             * Very important line!
             * This configure's Rhino to run in Interpretive mode.
             * This is fine because we're not compiling js to java, and without
             * this line you'll see a 64K error when loading large js files like extjs,
             * which is a Java issue.
             */
            context.setOptimizationLevel(-1);
            shell = new Shell();
            context.initStandardObjects(shell);

            // Define some global functions particular to the shell. Note
            // that these functions are not part of ECMA.
            String[] names = { "info", "debug", "error", "quit", "version", "load",
                    "assertEquals", "assertNotNull", "loadAllInDir",
                    "assertTrue", "assertFalse"};
            shell.defineFunctionProperties(names, Shell.class,
                                           ScriptableObject.DONTENUM);
            /**
             * Bootstrap loads envjs, ext
             */
            shell.processSource(context, "test/com/objectpartners/envjs/bootstrap.js");

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Be sure to close the context here.
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        Context.exit();
    }

    /**
     *
     * Convenience method to run a javascript test.
     *
     * @param fileName
     */
    protected void processScript(final String fileName) {
        shell.processSource(context, fileName);
    }

}
