/**
 * 
 */
package com.objectpartners.htmlunit;

import java.io.File;
import java.io.IOException;
import java.util.List;

import com.gargoylesoftware.htmlunit.BrowserVersion;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlButton;
import com.gargoylesoftware.htmlunit.html.HtmlDivision;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlPage;


/**
 * @author Jeff
 *
 */
public class DesktopTest extends AbstractHtmlUnitTest {

    private static final String HTMLUNIT_RESOURCE_DIR = "test/com/objectpartners/htmlunit/test/resource/desktop";

    /**
     * Basic file to use for tests.
     */
    private static final String TEMP_FILE = HTMLUNIT_RESOURCE_DIR + "/test.html";

	private static HtmlPage htmlPage;
	private static File htmlFile = new File(TEMP_FILE);
	
	@Override	
	protected void setUp() throws Exception {
		super.setUp();
        final WebClient webClient = new WebClient(BrowserVersion.FIREFOX_2);
        WebClient.setIgnoreOutsideContent(true);

        // throw an exception if we encounter any javascript errors
        webClient.setThrowExceptionOnScriptError(true);
        assertTrue(htmlFile.canRead());
        htmlPage = (HtmlPage) webClient.getPage(htmlFile.toURL());
        waitAfterPageLoad(htmlPage);		
	}
	
	public void testGetStartMenu() throws IOException {
		//Make sure the button is there, and if it is, click it
		List<HtmlButton> elements = (List<HtmlButton>) htmlPage.getByXPath("//*[contains(@class,'x-btn-text start')]");
		assertTrue(elements.size() == 1);
		HtmlButton button = elements.get(0);		
		button.click();
		
		// Get the start menu, and make sure the menu items are there
		List<HtmlDivision> startMenuElements = (List<HtmlDivision>) htmlPage.getByXPath("//*[contains(@class,'x-panel x-border-panel ux-start-menu-apps-panel')]");
		assertTrue(startMenuElements.size() == 1);
		HtmlDivision startMenuDiv = startMenuElements.get(0);
		Iterable<HtmlElement> childElements = startMenuDiv.getChildElements();
		assertNotNull(childElements);
		int count = 0;
		HtmlElement ulElement = null;
		for (HtmlElement htmlElement : childElements) {
			count++;
			ulElement = htmlElement;
		}
		assertNotNull(ulElement);		
		assertTrue(count == 1);		
	
		// get the list of submenus
		childElements = ulElement.getChildElements();
		count = 0;
		for (HtmlElement htmlElement : childElements) {
			count++;
		}
		assertTrue(count == 5);
	}

}
